@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2020 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2021 Live Networks, Inc.  All rights reserved.
 // A RTSP server
 // Implementation of functionality related to the "REGISTER" and "DEREGISTER" commands
 
@@ -326,10 +326,11 @@ RTSPServerWithREGISTERProxying* RTSPServerWithREGISTERProxying
 	    unsigned reclamationSeconds,
 	    Boolean streamRTPOverTCP, int verbosityLevelForProxying,
 	    char const* backEndUsername, char const* backEndPassword) {
-  int ourSocket = setUpOurSocket(env, ourPort);
-  if (ourSocket == -1) return NULL;
+  int ourSocketIPv4 = setUpOurSocket(env, ourPort, AF_INET);
+  int ourSocketIPv6 = setUpOurSocket(env, ourPort, AF_INET6);
+  if (ourSocketIPv4 < 0 && ourSocketIPv6 < 0) return NULL;
   
-  return new RTSPServerWithREGISTERProxying(env, ourSocket, ourPort,
+  return new RTSPServerWithREGISTERProxying(env, ourSocketIPv4, ourSocketIPv6, ourPort,
 					    authDatabase, authDatabaseForREGISTER,
 					    reclamationSeconds,
 					    streamRTPOverTCP, verbosityLevelForProxying,
@@ -337,12 +338,12 @@ RTSPServerWithREGISTERProxying* RTSPServerWithREGISTERProxying
 }
 
 RTSPServerWithREGISTERProxying
-::RTSPServerWithREGISTERProxying(UsageEnvironment& env, int ourSocket, Port ourPort,
+::RTSPServerWithREGISTERProxying(UsageEnvironment& env, int ourSocketIPv4, int ourSocketIPv6, Port ourPort,
 				 UserAuthenticationDatabase* authDatabase, UserAuthenticationDatabase* authDatabaseForREGISTER,
 				 unsigned reclamationSeconds,
 				 Boolean streamRTPOverTCP, int verbosityLevelForProxying,
 				 char const* backEndUsername, char const* backEndPassword)
-  : RTSPServer(env, ourSocket, ourPort, authDatabase, reclamationSeconds),
+  : RTSPServer(env, ourSocketIPv4, ourSocketIPv6, ourPort, authDatabase, reclamationSeconds),
     fStreamRTPOverTCP(streamRTPOverTCP), fVerbosityLevelForProxying(verbosityLevelForProxying),
     fRegisteredProxyCounter(0), fAllowedCommandNames(NULL), fAuthDBForREGISTER(authDatabaseForREGISTER),
     fBackEndUsername(strDup(backEndUsername)), fBackEndPassword(strDup(backEndPassword)) {
@@ -368,7 +369,7 @@ Boolean RTSPServerWithREGISTERProxying
 		      char const* proxyURLSuffix, char*& responseStr) {
   // First, check whether we have already proxied a stream as "proxyURLSuffix":
   if (proxyURLSuffix != NULL) {
-    ServerMediaSession* sms = lookupServerMediaSession(proxyURLSuffix);
+    ServerMediaSession* sms = getServerMediaSession(proxyURLSuffix);
     if ((strcmp(cmd, "REGISTER") == 0 && sms != NULL) ||
 	(strcmp(cmd, "DEREGISTER") == 0 && sms == NULL)) {
       responseStr = strDup("451 Invalid parameter");
@@ -420,7 +421,8 @@ void RTSPServerWithREGISTERProxying
     envir() << "\tPlay this stream using the URL: " << proxyStreamURL << "\n";
     delete[] proxyStreamURL;
   } else { // "DEREGISTER"
-    deleteServerMediaSession(lookupServerMediaSession(proxyStreamName));
+    //    deleteServerMediaSession(lookupServerMediaSession(proxyStreamName));
+    lookupServerMediaSession(proxyStreamName, &GenericMediaServer::deleteServerMediaSession);
   }
 }
 
