@@ -13,14 +13,13 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
-// Copyright (c) 1996-2020, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2021, Live Networks, Inc.  All rights reserved
 // A test program that receives a RTP/RTCP multicast MP3 stream,
 // and outputs the resulting MP3 file stream to 'stdout'
 // main program
 
 #include "liveMedia.hh"
 #include "GroupsockHelper.hh"
-
 #include "BasicUsageEnvironment.hh"
 
 // To receive a stream of 'ADUs' rather than raw MP3 frames, uncomment this:
@@ -68,16 +67,19 @@ int main(int argc, char** argv) {
   const unsigned char ttl = 1; // low, in case routers don't admin scope
 #endif
 
-  struct in_addr sessionAddress;
-  (void)inet_pton(AF_INET, sessionAddressStr, &sessionAddress.s_addr);
+  NetAddressList sessionAddresses(sessionAddressStr);
+  struct sockaddr_storage sessionAddress;
+  copyAddress(sessionAddress, sessionAddresses.firstAddress());
+
   const Port rtpPort(rtpPortNum);
   const Port rtcpPort(rtcpPortNum);
 
 #ifdef USE_SSM
-  char* sourceAddressStr = "aaa.bbb.ccc.ddd";
+  char const* sourceAddressStr = "aaa.bbb.ccc.ddd";
                            // replace this with the real source address
-  struct in_addr sourceFilterAddress;
-  (void)inet_pton(AF_INET, sourceAddressStr, &sourceFilterAddress.s_addr);
+  NetAddressList sourceFilterAddresses(sourceAddressStr);
+  struct sockaddr_storage sourceFilterAddress;
+  copyAddress(sourceFilterAddress, sourceFilterAddresses.firstAddress());
 
   Groupsock rtpGroupsock(*env, sessionAddress, sourceFilterAddress, rtpPort);
   Groupsock rtcpGroupsock(*env, sessionAddress, sourceFilterAddress, rtcpPort);
